import { DATASET_ENDPOINT, getBaseURL } from "./constants.js";

export default class Dataset {
    static #datasets = new Map();

    #shortDescription;
    #author;
    #category;
    #date;
    #fullDescription;
    #id;
    #rating;
    #title;
    #type;
    #upvotes;
    #url;
    #votes;
    #license;
    #termsOfUse;
    #elements = [];

    static get(id) {
        return this.#datasets.get(id);
    }

    constructor({ abst: shortDescription, author, categorie, date, description, id, raiting, title, type, upvotes, url, votes, license, termsOfUse }) {
        this.#shortDescription = shortDescription;
        this.#author = author;
        this.#category = categorie;
        this.#date = date;
        this.#fullDescription = description;
        this.#id = id;
        this.#rating = raiting;
        this.#title = title;
        this.#type = type;
        this.#upvotes = upvotes;
        this.#url = url;
        this.#votes = votes;
        this.#license = license;
        this.#termsOfUse = termsOfUse;

        Dataset.#datasets.set(id, this);
    }

    // Getters
    get shortDescription() {
        return this.#shortDescription;
    }

    get author() {
        return this.#author;
    }

    get category() {
        return this.#category;
    }

    get date() {
        return this.#date;
    }

    get fullDescription() {
        return this.#fullDescription;
    }

    get id() {
        return this.#id;
    }

    get rating() {
        return this.#rating;
    }

    get title() {
        return this.#title;
    }

    get type() {
        return this.#type;
    }

    get upvotes() {
        return this.#upvotes;
    }

    get url() {
        return this.#url;
    }

    get votes() {
        return this.#votes;
    }

    get license() {
        return this.#license;
    }

    get termsOfUse() {
        return this.#termsOfUse;
    }

    get mainElement() {
        return this.#elements[0];
    }

    get elements() {
        return this.#elements;
    }

    parseDate() {
        return new Date(this.#date);
    }

    // Main methods

    // Only on main page
    createDatasetHTMLElement() {
        const clone = this.#createTemplateInstance("dataset-template");
        if (clone == null) {
            return null;
        }

        clone.querySelector(".dataset").dataset.id = this.#id;
        clone.querySelector(".dataset-title").innerText = this.#title;
        clone.querySelector(".dataset-description").innerText = this.#shortDescription;
        clone.querySelector(".upvote-count").innerText = this.#upvotes;

        this.#elements.push(clone.children[0]);
        this.#createUpvoteButtons(clone);

        return clone;
    }

    // Only on details page
    createUpvoteComponent() {
        let clone = this.#createTemplateInstance("voting-template")
        
        clone.querySelector(".upvote-count").innerText = this.#upvotes;

        this.#elements.push(clone.children[0]);
        this.#createUpvoteButtons(clone);

        return clone;
    }

    #createUpvoteButtons(templateInstance) {
        if (this.storageGetKey("is-voted", false)) {
            // The template instance (clone) has to be pushed before this can work
            this.#disableVoteBtns();
        }

        // Event Listeners
        templateInstance.querySelector(".upvote-btn").addEventListener("click", () => {
            this.vote();
        });

        templateInstance.querySelector(".downvote-btn").addEventListener("click", () => {
            this.vote(false);
        })
    }

    #createTemplateInstance(templateId) {
        let template = document.getElementById(templateId);
        if (template == null) {
            return null;
        }

        return template.content.cloneNode(true);
    }

    isMainElement(element) {
        return this.#elements.length > 0 && this.#elements[0].isEqualNode(element);
    }

    async vote(up = true) {
        const fetchURL = new URL(
            `${DATASET_ENDPOINT}/id/${this.#id}/${up ? "up" : "down"}vote`,
            getBaseURL(),
        );

        const response = await fetch(fetchURL, {
            method: "PUT",
            headers: {
                'Content-Type': 'application/json',
            }
        });
        const data = await response.json();

        this.#upvotes = data.upvotes;
        for (const element of this.#elements) {
            element.querySelector(".upvote-count").innerText = this.#upvotes;
        }

        this.storageSetKey("vote-type", up ? "upvote" : "downvote");
        this.storageSetKey("is-voted", true);
        this.#disableVoteBtns();
    }

    #disableVoteBtns() {
        if (this.#elements.length > 0) {
            const voteType = this.storageGetKey("vote-type", null);

            this.#elements.flatMap(e => Array.from(e.querySelectorAll(".upvote .btn")))
                .forEach(btn => {
                    btn.disabled = true;

                    if (btn.classList.contains(`${voteType}-btn`)) {
                        btn.classList.add("isVoted");
                    }
                });
        }
    }

    storageGet() {
        const json = localStorage.getItem(this.#id);
        return json != null ? JSON.parse(json) : {};
    }

    storageSetKey(key, value) {
        let currentStorage = this.storageGet();
        currentStorage[key] = value;
        localStorage.setItem(this.#id, JSON.stringify(currentStorage));
    }

    storageGetKey(key, defaultValue) {
        return this.storageGet()?.[key] ?? defaultValue;
    }
}
